package org.bitbucket.bradleysmithllc.etlunit;

import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestClass;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;

import java.util.HashMap;
import java.util.Map;

public class MapLocal extends ThreadLocal<Map<String, Object>>
{
	@Override
	protected Map<String, Object> initialValue()
	{
		return new HashMap<String, Object>();
	}

	public <T extends Object> T get(String name, Class<T> type)
	{
		return get(name, type, true);
	}

	public <T extends Object> T get(String name, Class<T> type, boolean optional)
	{
		Object ret = get().get(name);

		if (ret == null)
		{
			if (optional)
			{
				return null;
			}

			throw new IllegalArgumentException("Object named '" + name + "' not found in map");
		}

		if (!type.isAssignableFrom(ret.getClass()))
		{
			throw new ClassCastException("Stored object type does not match requested type");
		}

		return (T) ret;
	}

	public <T extends Object> void set(String name, T type)
	{
		get().put(name, type);
	}

	public void setCurrentlyProcessingTestClass(ETLTestClass etlClass)
	{
		set("currentlyProcessingTestClass", etlClass);
	}

	public ETLTestClass getCurrentlyProcessingTestClass()
	{
		return get("currentlyProcessingTestClass", ETLTestClass.class);
	}

	public void setCurrentlyProcessingTestMethod(ETLTestMethod etlClass)
	{
		set("currentlyProcessingTestMethod", etlClass);
	}

	public ETLTestMethod getCurrentlyProcessingTestMethod()
	{
		return get("currentlyProcessingTestMethod", ETLTestMethod.class);
	}

	public void setCurrentlyProcessingMethod(ETLTestMethod etlClass)
	{
		set("currentlyProcessingMethod", etlClass);
	}

	public ETLTestMethod getCurrentlyProcessingMethod()
	{
		return get("currentlyProcessingMethod", ETLTestMethod.class);
	}

	public void setCurrentlyProcessingTestOperation(ETLTestOperation etlClass)
	{
		set("currentlyProcessingTestOperation", etlClass);
	}

	public ETLTestOperation getCurrentlyProcessingTestOperation()
	{
		return get("currentlyProcessingTestOperation", ETLTestOperation.class);
	}
}